package lcsb.mapviewer.api;

import java.io.IOException;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;

public abstract class BaseController {
	private Logger			 logger	= Logger.getLogger(BaseController.class);

	private ObjectMapper mapper	= new ObjectMapper();

	@ExceptionHandler({ Exception.class })
	public ResponseEntity<Object> handleException(Exception e, WebRequest request) {
		if (e instanceof lcsb.mapviewer.services.SecurityException) {
			return new ResponseEntity<Object>("{\"error\" : \"Access denied.\",\"reason\":\"" + e.getMessage() + "\"}", new HttpHeaders(), HttpStatus.FORBIDDEN);
		} else if (e instanceof ObjectNotFoundException) {
			return new ResponseEntity<Object>("{\"error\" : \"Object not found.\",\"reason\":\"" + e.getMessage() + "\"}", new HttpHeaders(), HttpStatus.NOT_FOUND);
		} else if (e instanceof QueryException) {
			return new ResponseEntity<Object>(
					"{\"error\" : \"Query server error.\",\"reason\":\"" + e.getMessage() + "\"}", new HttpHeaders(), HttpStatus.BAD_REQUEST);
		} else {
			logger.error(e, e);
			return new ResponseEntity<Object>(
					"{\"error\" : \"Internal server error.\",\"reason\":\"" + e.getMessage() + "\"}", new HttpHeaders(), HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	protected Map<String, Object> parseBody(String body) throws IOException, JsonParseException, JsonMappingException {
		logger.debug("Parse body: " + body);
		ObjectNode result = mapper.readValue(body, ObjectNode.class);
		return mapper.convertValue(result, Map.class);
	}

	protected Map<String, Object> getData(Map<String, Object> node, String objectName) {
		return (Map<String, Object>) node.get(objectName);
	}

}
