package lcsb.mapviewer.api;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.common.exception.InvalidStateException;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;

@Transactional(value = "txManager")
public abstract class BaseRestImpl {
	@Autowired
	private IModelService	modelService;
	@Autowired
	private IUserService	userService;

	protected Map<String, Object> okStatus() {
		Map<String, Object> result = new HashMap<>();
		return result;
	}

	protected Map<String, Object> createMinifiedSearchResult(Object object) {
		Map<String, Object> result = new HashMap<>();
		if (object instanceof Element) {
			result.put("type", ElementIdentifierType.ALIAS);
			Element element = (Element) object;
			result.put("id", element.getId());
			result.put("modelId", element.getModel().getId());
		} else if (object instanceof Reaction) {
			result.put("type", ElementIdentifierType.REACTION);
			Reaction element = (Reaction) object;
			result.put("id", element.getId());
			result.put("modelId", element.getModel().getId());

		} else {
			throw new InvalidStateException("Unknown type of result: " + object.getClass());
		}
		return result;
	};

	protected List<Model> getModels(String projectId, String modelId, String token) throws SecurityException {
		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));
		List<Model> models = new ArrayList<>();

		if (!modelId.equals("*")) {
			for (String str : modelId.split(",")) {
				models.add(model.getSubmodelById(Integer.valueOf(str)));
			}
		} else {
			models.addAll(model.getSubmodels());
			models.add(model);
		}
		return models;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

}
