package lcsb.mapviewer.api.projects.models.publications;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.exception.InvalidStateException;
import lcsb.mapviewer.model.map.AnnotatedObject;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.model.ModelSubmodelConnection;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.ILayoutService;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.ISearchService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;
import lcsb.mapviewer.services.view.AnnotationViewFactory;
import lcsb.mapviewer.services.view.AuthenticationToken;
import lcsb.mapviewer.services.view.OverviewImageViewFactory;

@Transactional(value = "txManager")
public class PublicationsRestImpl extends BaseRestImpl{

	Logger													 logger	= Logger.getLogger(PublicationsRestImpl.class);

	@Autowired
	private IUserService						 userService;

	@Autowired
	private IModelService						 modelService;

	@Autowired
	private ISearchService					 searchService;

	@Autowired
	AnnotationViewFactory						 annotationViewFactory;

	@Autowired
	private PubmedParser						 pubmedParser;

	@Autowired
	private OverviewImageViewFactory factory;

	public SortedMap<MiriamData, List<AnnotatedObject>> getPublications(List<Model> models) {
		SortedMap<MiriamData, List<AnnotatedObject>> publications = new TreeMap<>();
		for (Model modelData : models) {
			for (Element element : modelData.getElements()) {
				for (MiriamData md : element.getMiriamData()) {
					if (md.getDataType().equals(MiriamType.PUBMED)) {
						List<AnnotatedObject> list = publications.get(md);
						if (list == null) {
							list = new ArrayList<>();
							publications.put(md, list);
						}
						list.add(element);
					}
				}
			}
			for (Reaction reaction : modelData.getReactions()) {
				for (MiriamData md : reaction.getMiriamData()) {
					if (md.getDataType().equals(MiriamType.PUBMED)) {
						List<AnnotatedObject> list = publications.get(md);
						if (list == null) {
							list = new ArrayList<>();
							publications.put(md, list);
						}
						list.add(reaction);
					}
				}

			}
		}
		return publications;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the searchService
	 * @see #searchService
	 */
	public ISearchService getSearchService() {
		return searchService;
	}

	/**
	 * @param searchService
	 *          the searchService to set
	 * @see #searchService
	 */
	public void setSearchService(ISearchService searchService) {
		this.searchService = searchService;
	}

	/**
	 * @return the factory
	 * @see #factory
	 */
	public OverviewImageViewFactory getFactory() {
		return factory;
	}

	/**
	 * @param factory
	 *          the factory to set
	 * @see #factory
	 */
	public void setFactory(OverviewImageViewFactory factory) {
		this.factory = factory;
	}

	public Map<String, Object> getPublications(String projectId, String modelId, String token, String startString, Integer length) throws SecurityException, QueryException {
		List<Model> models = getModels(projectId, modelId, token);

		Integer start = Math.max(0, Integer.valueOf(startString));
		List<Map<String, Object>> resultList = new ArrayList<>();

		SortedMap<MiriamData, List<AnnotatedObject>> publications = getPublications(models);
		int index = 0;
		Set<String> columns = new HashSet<>();
		columns.add("id");
		columns.add("type");
		columns.add("modelid");
		for (Map.Entry<MiriamData, List<AnnotatedObject>> entry : publications.entrySet()) {
			if (index >= start && index < start + length) {
				List<Object> elements = new ArrayList<>();
				for (AnnotatedObject object : entry.getValue()) {
					elements.add(createMinifiedSearchResult(object));
				}

				Map<String, Object> row = new HashMap<>();
				row.put("elements", elements);
				if (entry.getKey().getDataType().equals(MiriamType.PUBMED)) {
					try {
						row.put("publication", pubmedParser.getPubmedArticleById(Integer.valueOf(entry.getKey().getResource())));
					} catch (Exception e) {
						logger.error(e, e);
						row.put("publication", annotationViewFactory.create(entry.getKey()));
					}
				} else {
					row.put("publication", annotationViewFactory.create(entry.getKey()));
				}
				resultList.add(row);
			}
			index++;

		}

		Map<String, Object> result = new HashMap<>();
		result.put("data", resultList);
		result.put("totalSize", publications.size());
		result.put("start", start);
		result.put("length", resultList.size());
		return result;
	}

	/**
	 * @return the pubmedParser
	 * @see #pubmedParser
	 */
	public PubmedParser getPubmedParser() {
		return pubmedParser;
	}

	/**
	 * @param pubmedParser
	 *          the pubmedParser to set
	 * @see #pubmedParser
	 */
	public void setPubmedParser(PubmedParser pubmedParser) {
		this.pubmedParser = pubmedParser;
	}

}
