package lcsb.mapviewer.api.projects;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.converter.graphics.PdfImageGenerator;
import lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.services.interfaces.IModelService;

public class ProjectRestImplTest extends RestTestFunctions {
	Logger					logger = Logger.getLogger(ProjectRestImplTest.class);

	@Autowired
	ProjectRestImpl	_projectRestImpl;

	@Autowired
	IModelService		modelService;

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGetModelAsImage() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			projectRest.getModelAsImage(token.getId(), "sample", "0", "", "", "", "", "");
			fail("Exception expected");
		} catch (InvalidArgumentException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetModelAsImage2() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			projectRest.getModelAsImage(token.getId(), "sample", "0", PdfImageGenerator.class.getCanonicalName(), "", "", "", "");
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetModelAsFileModel() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			projectRest.getModelAsModelFile(token.getId(), "sample", "0", "", "", "", "", "");
			fail("Exception expected");
		} catch (InvalidArgumentException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetModelAsFileModel2() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			projectRest.getModelAsModelFile(token.getId(), "sample", "0", CellDesignerXmlParser.class.getCanonicalName(), "", "", "", "0,0;90,0;90,90;90,0");
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetModelAsFileModel3() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			projectRest.getModelAsModelFile(token.getId(), "sample", "0", "", "", "", "", "0,0;90,0;90,90;90,0");
			fail("Exception expected");
		} catch (QueryException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetMetaData() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			ProjectMetaData result = projectRest.getMetaData("sample", token.getId());
			Gson gson = new Gson();
			assertNotNull(gson.toJson(result));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetMetaDataForComplexWithImages() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/complex_model_with_submaps.zip");
			ProjectMetaData result = projectRest.getMetaData("sample", token.getId());
			Gson gson = new Gson();
			assertNotNull(gson.toJson(result));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	private ProjectRestImpl createMockProjectRest(String string) throws Exception {
		Model model = super.getModelForFile(string, true);
		IModelService mockModelService = Mockito.mock(IModelService.class);
		Mockito.when(mockModelService.getLastModelByProjectId(anyString(), any())).thenReturn(model);
		_projectRestImpl.setModelService(mockModelService);
		return _projectRestImpl;
	}

}
