package lcsb.mapviewer.api.genomics;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.layout.ReferenceGenomeType;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IReferenceGenomeService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.view.AuthenticationToken;
import lcsb.mapviewer.services.view.ReferenceGenomeView;

@Transactional(value = "txManager")
public class ReferenceGenomeRestImpl {

	private Logger									logger = Logger.getLogger(ReferenceGenomeRestImpl.class);

	@Autowired
	private IUserService						userService;

	/**
	 * Service that manages reference genomes.
	 */
	@Autowired
	private IReferenceGenomeService	referenceGenomeService;

	public ReferenceGenomeView getReferenceGenome(String token, String organismId, String type, String version) throws SecurityException, QueryException {
		AuthenticationToken authenticationToken = userService.getToken(token);

		MiriamData organism = null;
		if (organismId != null && !organismId.isEmpty()) {
			organism = new MiriamData(MiriamType.TAXONOMY, organismId);
		} else {
			throw new QueryException("Unknown taxonomy organism: " + organismId);
		}
		ReferenceGenomeView result = null;
		try {
			ReferenceGenomeType genomeType = ReferenceGenomeType.valueOf(type);
			version = version.replaceAll("\\*", "");
			result = referenceGenomeService.getReferenceGenomeViewByParams(organism, genomeType, version, authenticationToken);
			if (result == null) {
				throw new QueryException("Cannot find requested reference genome");
			}
		} catch (IllegalArgumentException e) {
			throw new QueryException("Cannot find type: " + type);
		}
		return result;
	}

}
